@@ -60,7 +60,7 @@ sub import {
 }
 
 
-our $VERSION = '0.0243';    # VERSION
+our $VERSION = '0.0244';    # VERSION
 
 1;
 
@@ -78,7 +78,7 @@ Evo - Evo - the next generation development framework
 
 =head1 VERSION
 
-version 0.0243
+version 0.0244
 
 =head1 DESCRIPTION
 
@@ -112,16 +112,18 @@ It provides rewritten and postmodern features like
   # enables strict, warnings, utf8, :5.22, signatures, postderef
   use Evo;
 
+=head1 XS
+
+This module ships with optional C parts for performance. You can avoid installing them by providing PUREPERL_ONLY environmental variable
+
+  PUREPERL_ONLY=1 cpanm Evo
+
 =head1 STATE
 
 This module is under active development. It changes often and a lot! Get involved L<https://github.com/alexbyk/perl-evo>
 
 Also there are many gaps in documentation.
 
-=head1 XS
-
-You can get a little bit faster classes by installing C<Evo::XS>. You will need a C compiler. See L<https://github.com/alexbyk/perl-evo-xs>
-
 =head1 VIM
 
 =for HTML <p><img src="https://raw.github.com/alexbyk/perl-evo/master/demo.gif" alt="Perl Evo gif" /></p>
